'use strict';

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _WeakMap = require('babel-runtime/core-js/weak-map')['default'];

var _Object$assign = require('babel-runtime/core-js/object/assign')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _reactRelay = require('react-relay');

var _reactRelay2 = _interopRequireDefault(_reactRelay);

var _getParamsForRoute = require('./getParamsForRoute');

var _getParamsForRoute2 = _interopRequireDefault(_getParamsForRoute);

var RouteAggregator = (function () {
  function RouteAggregator() {
    _classCallCheck(this, RouteAggregator);

    this._uniqueQueryNames = new _WeakMap();
    this._lastQueryIndex = 0;

    this.route = null;
    this._fragmentSpecs = null;

    this._data = {};
    this._failure = null;
  }

  RouteAggregator.prototype.updateRoute = function updateRoute(routerProps) {
    var _this = this;

    var branch = routerProps.branch;
    var params = routerProps.params;
    var location = routerProps.location;

    var relayRoute = {
      name: null,
      queries: {},
      params: {}
    };
    var fragmentSpecs = {};

    branch.forEach(function (route) {
      var queries = route.queries;

      if (!queries) {
        return;
      }

      var component = route.component;

      // In principle not all container component routes have to specify
      // queries, because some of them might somehow receive fragments from
      // their parents, but it would definitely be wrong to specify queries
      // for a component that isn't a container.
      _invariant2['default'](_reactRelay2['default'].isContainer(component), 'relay-nested-routes: Route with queries specifies component `%s` ' + 'that is not a Relay container.', component.displayName || component.name);

      var routeParams = _getParamsForRoute2['default']({ route: route, branch: branch, params: params, location: location });
      _Object$assign(relayRoute.params, routeParams);

      _Object$keys(queries).forEach(function (queryName) {
        var query = queries[queryName];
        var uniqueQueryName = _this._getUniqueQueryName(query, queryName);

        relayRoute.queries[uniqueQueryName] = function () {
          return query(component, routeParams);
        };
        fragmentSpecs[uniqueQueryName] = { component: component, queryName: queryName };
      });
    });

    relayRoute.name = ['$$_aggregated'].concat(_Object$keys(relayRoute.queries)).join('-');

    // RootContainer uses referential equality to check for route change, so
    // replace the route object entirely.
    this.route = relayRoute;
    this._fragmentSpecs = fragmentSpecs;
  };

  RouteAggregator.prototype._getUniqueQueryName = function _getUniqueQueryName(query, queryName) {
    var uniqueQueryName = this._uniqueQueryNames.get(query);
    if (uniqueQueryName === undefined) {
      uniqueQueryName = '$$_' + queryName + '_' + ++this._lastQueryIndex;
      this._uniqueQueryNames.set(query, uniqueQueryName);
    }

    return uniqueQueryName;
  };

  RouteAggregator.prototype.setLoading = function setLoading() {
    this._failure = null;
  };

  RouteAggregator.prototype.setFetched = function setFetched(data) {
    this._failure = null;
    this._data = data;
  };

  RouteAggregator.prototype.setFailure = function setFailure(error, retry) {
    this._failure = [error, retry];
  };

  RouteAggregator.prototype.getData = function getData(queries, params) {
    // Check that the subset of parameters used for this route match those used
    // for the fetched data.
    for (var _iterator = _Object$keys(params), _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _getIterator(_iterator);;) {
      var _ref;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref = _i.value;
      }

      var paramName = _ref;

      if (this._data[paramName] !== params[paramName]) {
        return this._getDataNotFound();
      }
    }

    var fragmentPointers = {};
    for (var _iterator2 = _Object$keys(queries), _isArray2 = Array.isArray(_iterator2), _i2 = 0, _iterator2 = _isArray2 ? _iterator2 : _getIterator(_iterator2);;) {
      var _ref2;

      if (_isArray2) {
        if (_i2 >= _iterator2.length) break;
        _ref2 = _iterator2[_i2++];
      } else {
        _i2 = _iterator2.next();
        if (_i2.done) break;
        _ref2 = _i2.value;
      }

      var queryName = _ref2;

      var query = queries[queryName];
      var uniqueQueryName = this._getUniqueQueryName(query, queryName);

      var fragmentPointer = this._data[uniqueQueryName];
      if (!fragmentPointer) {
        return this._getDataNotFound();
      }

      fragmentPointers[queryName] = fragmentPointer;
    }

    return { fragmentPointers: fragmentPointers };
  };

  RouteAggregator.prototype._getDataNotFound = function _getDataNotFound() {
    return { failure: this._failure };
  };

  RouteAggregator.prototype.getFragmentNames = function getFragmentNames() {
    return _Object$keys(this._fragmentSpecs);
  };

  RouteAggregator.prototype.getFragment = function getFragment(fragmentName, variableMapping) {
    var _fragmentSpecs$fragmentName = this._fragmentSpecs[fragmentName];
    var component = _fragmentSpecs$fragmentName.component;
    var queryName = _fragmentSpecs$fragmentName.queryName;

    return component.getFragment(queryName, variableMapping);
  };

  // TODO: Remove when Relay>0.1.1 is released (facebook/relay#103), since
  // getFragmentNames and getFragment fulfill the isContainer contract.

  RouteAggregator.prototype.getQuery = function getQuery() {};

  RouteAggregator.prototype.getQueryNames = function getQueryNames() {};

  return RouteAggregator;
})();

exports['default'] = RouteAggregator;
module.exports = exports['default'];